/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/DateTime.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/s3tables/S3Tables_EXPORTS.h>
#include <aws/s3tables/model/TableType.h>

#include <utility>

namespace Aws {
namespace Utils {
namespace Json {
class JsonValue;
class JsonView;
}  // namespace Json
}  // namespace Utils
namespace S3Tables {
namespace Model {

/**
 * <p>Contains details about a table.</p><p><h3>See Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/s3tables-2018-05-10/TableSummary">AWS
 * API Reference</a></p>
 */
class TableSummary {
 public:
  AWS_S3TABLES_API TableSummary() = default;
  AWS_S3TABLES_API TableSummary(Aws::Utils::Json::JsonView jsonValue);
  AWS_S3TABLES_API TableSummary& operator=(Aws::Utils::Json::JsonView jsonValue);
  AWS_S3TABLES_API Aws::Utils::Json::JsonValue Jsonize() const;

  ///@{
  /**
   * <p>The name of the namespace.</p>
   */
  inline const Aws::Vector<Aws::String>& GetNamespace() const { return m_namespace; }
  inline bool NamespaceHasBeenSet() const { return m_namespaceHasBeenSet; }
  template <typename NamespaceT = Aws::Vector<Aws::String>>
  void SetNamespace(NamespaceT&& value) {
    m_namespaceHasBeenSet = true;
    m_namespace = std::forward<NamespaceT>(value);
  }
  template <typename NamespaceT = Aws::Vector<Aws::String>>
  TableSummary& WithNamespace(NamespaceT&& value) {
    SetNamespace(std::forward<NamespaceT>(value));
    return *this;
  }
  template <typename NamespaceT = Aws::String>
  TableSummary& AddNamespace(NamespaceT&& value) {
    m_namespaceHasBeenSet = true;
    m_namespace.emplace_back(std::forward<NamespaceT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The name of the table.</p>
   */
  inline const Aws::String& GetName() const { return m_name; }
  inline bool NameHasBeenSet() const { return m_nameHasBeenSet; }
  template <typename NameT = Aws::String>
  void SetName(NameT&& value) {
    m_nameHasBeenSet = true;
    m_name = std::forward<NameT>(value);
  }
  template <typename NameT = Aws::String>
  TableSummary& WithName(NameT&& value) {
    SetName(std::forward<NameT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The type of the table.</p>
   */
  inline TableType GetType() const { return m_type; }
  inline bool TypeHasBeenSet() const { return m_typeHasBeenSet; }
  inline void SetType(TableType value) {
    m_typeHasBeenSet = true;
    m_type = value;
  }
  inline TableSummary& WithType(TableType value) {
    SetType(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The Amazon Resource Name (ARN) of the table.</p>
   */
  inline const Aws::String& GetTableARN() const { return m_tableARN; }
  inline bool TableARNHasBeenSet() const { return m_tableARNHasBeenSet; }
  template <typename TableARNT = Aws::String>
  void SetTableARN(TableARNT&& value) {
    m_tableARNHasBeenSet = true;
    m_tableARN = std::forward<TableARNT>(value);
  }
  template <typename TableARNT = Aws::String>
  TableSummary& WithTableARN(TableARNT&& value) {
    SetTableARN(std::forward<TableARNT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The date and time the table was created at.</p>
   */
  inline const Aws::Utils::DateTime& GetCreatedAt() const { return m_createdAt; }
  inline bool CreatedAtHasBeenSet() const { return m_createdAtHasBeenSet; }
  template <typename CreatedAtT = Aws::Utils::DateTime>
  void SetCreatedAt(CreatedAtT&& value) {
    m_createdAtHasBeenSet = true;
    m_createdAt = std::forward<CreatedAtT>(value);
  }
  template <typename CreatedAtT = Aws::Utils::DateTime>
  TableSummary& WithCreatedAt(CreatedAtT&& value) {
    SetCreatedAt(std::forward<CreatedAtT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The date and time the table was last modified at.</p>
   */
  inline const Aws::Utils::DateTime& GetModifiedAt() const { return m_modifiedAt; }
  inline bool ModifiedAtHasBeenSet() const { return m_modifiedAtHasBeenSet; }
  template <typename ModifiedAtT = Aws::Utils::DateTime>
  void SetModifiedAt(ModifiedAtT&& value) {
    m_modifiedAtHasBeenSet = true;
    m_modifiedAt = std::forward<ModifiedAtT>(value);
  }
  template <typename ModifiedAtT = Aws::Utils::DateTime>
  TableSummary& WithModifiedAt(ModifiedAtT&& value) {
    SetModifiedAt(std::forward<ModifiedAtT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The Amazon Web Services service managing this table, if applicable. For
   * example, a replicated table is managed by the S3 Tables replication service.</p>
   */
  inline const Aws::String& GetManagedByService() const { return m_managedByService; }
  inline bool ManagedByServiceHasBeenSet() const { return m_managedByServiceHasBeenSet; }
  template <typename ManagedByServiceT = Aws::String>
  void SetManagedByService(ManagedByServiceT&& value) {
    m_managedByServiceHasBeenSet = true;
    m_managedByService = std::forward<ManagedByServiceT>(value);
  }
  template <typename ManagedByServiceT = Aws::String>
  TableSummary& WithManagedByService(ManagedByServiceT&& value) {
    SetManagedByService(std::forward<ManagedByServiceT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The unique identifier for the namespace that contains this table.</p>
   */
  inline const Aws::String& GetNamespaceId() const { return m_namespaceId; }
  inline bool NamespaceIdHasBeenSet() const { return m_namespaceIdHasBeenSet; }
  template <typename NamespaceIdT = Aws::String>
  void SetNamespaceId(NamespaceIdT&& value) {
    m_namespaceIdHasBeenSet = true;
    m_namespaceId = std::forward<NamespaceIdT>(value);
  }
  template <typename NamespaceIdT = Aws::String>
  TableSummary& WithNamespaceId(NamespaceIdT&& value) {
    SetNamespaceId(std::forward<NamespaceIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The unique identifier for the table bucket that contains this table.</p>
   */
  inline const Aws::String& GetTableBucketId() const { return m_tableBucketId; }
  inline bool TableBucketIdHasBeenSet() const { return m_tableBucketIdHasBeenSet; }
  template <typename TableBucketIdT = Aws::String>
  void SetTableBucketId(TableBucketIdT&& value) {
    m_tableBucketIdHasBeenSet = true;
    m_tableBucketId = std::forward<TableBucketIdT>(value);
  }
  template <typename TableBucketIdT = Aws::String>
  TableSummary& WithTableBucketId(TableBucketIdT&& value) {
    SetTableBucketId(std::forward<TableBucketIdT>(value));
    return *this;
  }
  ///@}
 private:
  Aws::Vector<Aws::String> m_namespace;

  Aws::String m_name;

  TableType m_type{TableType::NOT_SET};

  Aws::String m_tableARN;

  Aws::Utils::DateTime m_createdAt{};

  Aws::Utils::DateTime m_modifiedAt{};

  Aws::String m_managedByService;

  Aws::String m_namespaceId;

  Aws::String m_tableBucketId;
  bool m_namespaceHasBeenSet = false;
  bool m_nameHasBeenSet = false;
  bool m_typeHasBeenSet = false;
  bool m_tableARNHasBeenSet = false;
  bool m_createdAtHasBeenSet = false;
  bool m_modifiedAtHasBeenSet = false;
  bool m_managedByServiceHasBeenSet = false;
  bool m_namespaceIdHasBeenSet = false;
  bool m_tableBucketIdHasBeenSet = false;
};

}  // namespace Model
}  // namespace S3Tables
}  // namespace Aws
